#
# * prod_coredumpcheck
# *
# * Copyright (c) 2015 Ferrari electronic AG
#
# "use strict"
_ = require("lodash")
fs = require 'fs'
exec = require("child_process").exec
logger = require("../../../shared_modules/logger")
path = require 'path'



Producer = require "../../ProducerClass"

isWatching = false
watchDir4 = "/usr/fb/bin/4/sipisdn/"
watchDir5 = "/var/lib/systemd/coredump/"

class Coredumpcheck extends Producer
  constructor: ->
    super "coredumpcheck", 10 * 1000

    dumpFiles = []

    allDumpFiles = []
    getDumpFiles = (callback) =>
      fs.readdir watchDir4, (error, files) =>
        logger.error error if error
        allDumpFiles = ({name: fileName, time: (new Date()).toString()} for fileName in files when fileName.lastIndexOf("xcore", 0) is 0)
        callback() if callback
    getDumpFiles = (callback) =>
      fs.readdir watchDir5, (error, files) =>
        logger.error error if error
        allDumpFiles = ({name: fileName, time: (new Date()).toString()} for fileName in files when fileName.lastIndexOf("xcore", 0) is 0)
        callback() if callback


    send = =>
      @sendData allDumpFiles, true

    throttledGetAndSend = _.throttle(->
      getDumpFiles send
    , 1000)

    @start = =>
      #initially get dumpFiles and send them
      getDumpFiles send

      #reset watcher4 and start watching for changes
      watcher4.close() if watcher4
      watcher4 = false
      watcher4 = fs.watch watchDir4, (event, filename) =>
        logger.silly "#{filename} changed: #{event}"

        # if a core dump file changed, get dump files and send them
        if filename.lastIndexOf("xcore", 0) is 0
          throttledGetAndSend()

      #reset watcher5 and start watching for changes
      watcher5.close() if watcher5
      watcher5 = false
      watcher5 = fs.watch watchDir5, (event, filename) =>
        logger.silly "#{filename} changed: #{event}"

        # if a core dump file changed, get dump files and send them
        if filename.lastIndexOf("xcore", 0) is 0
          throttledGetAndSend()

    @stop = =>
      isWatching = false
      watcher4.close() if watcher4
      watcher4 = false

      watcher5.close() if watcher5
      watcher5 = false

    @delete = (file) =>
      if file is "all"
        logger.silly "deleting all core dumps"
        fs.readdir watchDir4, (error, files) =>
          logger.error error if error
          allDumpFiles = (fileName for fileName in files when fileName.lastIndexOf("xcore", 0) is 0)
          _.each allDumpFiles, (fileName) =>
            fs.unlink "#{watchDir4}#{fileName}"
            @start()
        fs.readdir watchDir5, (error, files) =>
          logger.error error if error
          allDumpFiles = (fileName for fileName in files when fileName.lastIndexOf("xcore", 0) is 0)
          _.each allDumpFiles, (fileName) =>
            fs.unlink "#{watchDir5}#{fileName}"
            @start()
      else
        file = path.basename file # remove path sections from file for security reasons
        # delete if file exists
        logger.debug "deleting #{file}"
        fs.unlink "#{watchDir4}#{file}", (err)=>
          @start() unless err

        file = path.basename file # remove path sections from file for security reasons
        # delete if file exists
        logger.debug "deleting #{file}"
        fs.unlink "#{watchDir5}#{file}", (err)=>
          @start() unless err


    @getStatus = (callback) =>
      if callback
        callback null, allDumpFiles

  subscribe: =>
    unless isWatching
      isWatching = true
      @start()
    super


module.exports = new Coredumpcheck()
